/*******************************************************************************
 * BATTERY CHARGER
 * Controls a sealed lead-acid battery charging
 * Author: George Novacek
 * Rev A: Start by verification of I/Os and the hardware
 * Rev B & C: Verify the ADC works with all 10 bits. It works with 8 bits only 
 * IMPORTANT - the spec states that for 10-bit ADC the ADCL must be read
 * first, then ADCH. This is NOT correct. It's the other way around!!!
 * Rev D: Put ADC acquisition into separate functions, moved all the functions
 * into a header file.
 * Rev E: Hardware/software integration. Tweaking the top voltage threshold.
 * Top voltage for the battery is now 910 count corresponding to 14.7V
 * Moving functions & definitions to other files.
 * Rev F: Adding leaking integration filter to the battery voltage acquisition.
 * Version F finished June 11, 2013
 ******************************************************************************/ 

#include <avr/io.h>
#include <util/delay.h>
#include "D:\Archives\Programs\ATTiny\Templates\ATTiny_Defines.h"
#include "D:\Archives\Programs\ATTiny\Templates\Charger_Function_Prototypes.h"
#include "D:\Archives\Programs\ATTiny\Templates\Charger_Functions.h"

uint8_t batMax = 910;                     // corresponds to 14.7V max

// PB0 - output. Switch to constant voltage charge when "1"  Pin #5
// PB1 - input. Battery being discharged when "1"            Pin #6
// PB2 - analog input. Battery voltage acquisition           Pin #7
// PB3 - output. RED LED                                     Pin #2
// PB4 - output. GREEN LED                                   Pin #3

// Modes: CHARGE - ORANGE, DISCHARGE - RED, STANDBY - GREEN

// =============================================================================
int main(void)
{ 
  DDRB = 0b11001;                         // set PB0 (pin5), PB3(pin 2), PB4
	                                      // pin 3) PB3 (pin 2) as outputs,
                                          // PB1 (pin6) and PB2 (Pin 7) as input
  initADC();                              // initiate ADC
  b0Low();                                // set constant current mode
  b3High();                               // RED light on
  b4High();                               // GREEN light on
    
  while (1)                               // loop forever
    {   
      readADC();                          // battery voltage
      if (batVolts<=910)                   // continue fast charge
        {
          b3High();                       // RED on
          b4High();                       // GREEN on
          b0Low();                        // constant current mode
          checkStatus();                  // charging or discharging?
          if (x >0)
            {
              disCharge();                // discharge mode  
            }
        }
        
      readADC();
      if(batVolts>910)
        {
          standBy();                      // stand-by floating mode
        }
    }
  return 0;
}
 